
html {
  font-family: Helvetica;
  background: url(/static/sd/images/gradient.png) repeat-x;
  background-color: #601;
  padding: 1em;

}
  

div.page {
    position: relative;
    max-width: 800px;
    min-width: 400px;
    background: #fff;
    margin: 0;
    padding: 0;
    margin-left: auto;
    margin-right: auto;
    padding-top: 1em;
    padding-left: 1em;
    padding-right: 1em;
    padding-bottom: 2em;
    -moz-border-radius: 1em;
    -webkit-border-radius: 1em;
}

div.body { 
    padding-top: 2em;
    margin-top: 2em;
    }

.logo {
    position: absolute;
    margin-left: -1em;
    margin-top: -1em;
    width: 70px;
   background-color: none;
   z-index: 50;
}


div.project-name {
    position: absolute;
    top: 0;
    right: 0;
    left: 0;
    padding-left: 70px;
    color: #fff;
    background-color: #000;
    display: inline;
    font-weight: bold;
    padding-top: 0.25em;
    padding-right: 0.5em;
    padding-bottom: 0.5em;
    -moz-border-radius-topright: 1em;
    -moz-border-radius-bottomright: 0em;
    -webkit-border-top-right-radius: 1em;
    -webkit-border-bottom-right-radius: 0em;
    z-index: 30;
}

h1 {
   position: absolute;
   left: 0;
   right: 0;
   top: 1.1em;
   height: 1em;
   display: block;
   overflow: hidden;
   text-overflow:ellipsis;
   padding-top:0.3em;
   padding-left: 70px;
   margin-bottom: 2.5em;
   background-color: #666;
   color: #fff;
   text-decoration: none;
   font-size: 1.6em;
   border-bottom: 0.25em solid #666;
   z-index: 0;
}

h2 {
   padding:  0.5em;
 text-decoration: none;
   font-weight: bold;
   font-size: 1.4em;

}

.log h2 {
    font-weight: normal;
}

.log h2 a.ticket-summary {
  font-weight: normal;
  text-decoration: none;
  color: #700;
} 

.log h2 a:hover {
    text-decoration: underline;
}


h2 .ticket-id {
    color: #ccc;
}


.log .nav {
 text-align: center;
padding-top: 0.25em;
padding-bottom: 0.25em;

}
.log .nav a:hover {
  text-decoration: underline;
  background: #aaa;
  color: #333;

}
.log .nav a { 
padding: 0.5em;
background: #ccc;
width: 5em;
display: inline-block;
text-align: center;
color: #666;
text-decoration: none;
}
.log .nav .prev { 
    -moz-border-radius-bottomleft: 0.5em;
    -webkit-border-bottom-left-radius: 0.5em;
    -moz-border-radius-topleft: 0.5em;
    -webkit-border-top-left-radius: 0.5em;

}
.log .nav .next {
    -moz-border-radius-bottomright: 0.5em;
    -webkit-border-bottom-right-radius: 0.5em;
    -moz-border-radius-topright: 0.5em;
    -webkit-border-top-right-radius: 0.5em;

}


ul.actions {
    align: center;
    display: block;
    min-width: 1em;
    max-width: 14em;
    margin-left: auto;
    margin-right: auto;
    padding: 0.25em;   
}

ul.actions li {
    list-style: none;
    border-right: 1px solid white;
    padding: 0.25em;
    display: inline;
    background: #ddd;
} 

ul.actions li:first-child {
    -moz-border-radius-topleft: 0.25em;
    -webkit-border-top-left-radius: 0.25em;
}

ul.actions li:last-child {
    -moz-border-radius-topright: 0.25em;
    -webkit-border-top-right-radius: 0.25em;
}


ul.actions li a { 
    text-decoration: none;
    color: #1133AA;
    padding: 0.5em;
    font-size: 0.8em;
}

ul.actions li:hover { 
    background: #ccc;
}

div.ticket_list ul li span {
   float: left;
   padding: 0.2em;
}

table.tablesorter thead th {
    color: #999;
    background-color: #fff;
}
table.tablesorter thead tr .header {
    background: none;
    border-bottom: 1px solid #666;
}
table.tablesorter thead tr .headerSortDown, 
table.tablesorter thead tr .headerSortUp, 
table.tablesorter thead tr th:hover {
    text-decoration: underline;
    background-color: #fff;
    color: #666;
}
table.tablesorter thead tr th {
    background-color: #fff;
}


th.headerSortUp,
th.headerSortDown {
    background: #fff;
}

    

ul.comments {
    list-style: none;
}

ul.comments span.metadata {
    color: #666; 

}

textarea:focus, input:focus { 
/*   padding: 2px;
   padding-left: 1px; */
   background-color: #ffc;
}

div.submit {
    width: auto;
    display: block;
    margin-top: 1em;
    margin-left: 2em;
    margin-right: 2em;
    text-align: right;
    padding-right: 1em;
}


input[type=submit] {
    background: #1133AA;
    color: #fff;
    margin: 0.5em;
    padding: 0.5em;
    top: 1em;
}
input[type=submit]:hover {
    background: #002299; 
}


textarea.prop-content {
    height: 10em;
    width: 40em;

}

label.prop-content {
    display: none;
}

div.widget label {
    display: inline-block;
    width: 8em;
    font-size: 0.8em;
    text-align: right;
    padding-right: 0.5em;
    color: #666;
    background-color: transparent;
    font-weight: bold;
    margin: 0;
    padding: 0;
    border: 0;
    padding-bottom: 0.8em;
}

div.widget div.value {
    display: inline-block;
    padding-top: 0.35em;
    padding-left: 0.6em;
    font-size: 0.95em;
}

div.widget input {
    margin-top: 0em;
    font-size: 0.95em;
    margin-left: 0.2em;
    padding: 0.2em;
}

div.comment-form {
    border: 1px solid #999;
    margin-left: 1em;
    margin-right: 1em;
    margin-top: 2em;
    padding: 2em;
    padding-top: 0;
    align: center;
    background: #ddd;
}

div.comment-form textarea {
    width: 100%;

}

div.comment-form h2 {
    margin-top: -1.5em;

}



div.widget {
    padding: 0.5em;
    padding-top: 0.6em;
    marging-bottom: 0.2em;
    margin-left: 1em;
    margin-right: 1em;
    height: 1.6em;
    border-bottom: 1px solid #ccc;
}

.widget {
    border-top: 1px solid #ccc;
}

.widget>.widget {
    border-top: none;
}

div.widget.status, div.widget.component, div.widget.milestone {
    display: inline-block;
    padding-top: 0.5em;
    padding-bottom: 1.75em;
    margin: 0;
    padding-left: 0.5em;
    padding-right: 1em;
    margin-right: -0.3em;
    width: 28%;
    border-bottom: none;
    border-top: none;
}

div.widget.description {
    display: block;
    height: auto;
}



div.widget.description .value {
    white-space: normal;
    display: block;
    overflow-x: auto;
}


div.widget.description textarea {
    width: 100%;
    font-size: 0.8em;
    height: 6em;
    padding-left: 0.5em;
    padding-top: 0.5em;
    padding-bottom: 0.5em;
}

div.widget.status {
    margin-left: 1em;
}

div.widget.status label, div.widget.component label, div.widget.milestone label, div.widget.description label
{
display: block;
text-align: left;
padding-left: 0.5em;
padding-bottom:0.25em;
}



div.widget.status input, 
div.widget.component input, 
div.widget.milestone input,
div.widget.status .value, 
div.widget.component .value, 
div.widget.milestone .value {

width: 100%;

}
/* 

.other-props { 
    margin-top: 0;
    padding-top: 0em;

}

.other-props .widget label,
.other-props .widget .label {
    width: 4em;
}
.other-props .widget {
    width: 40%;
}

.other-props .widget input {
    width: 13em;
}
*/

ul.page-nav {
    position: absolute;
    top: 0;
    right: 1.25em;
    font-size: 0.7em;
    z-index: 1000;
    color: #ccc;
    padding: 0;
}

ul.page-nav li, ul.page-nav li a {
    font-weight: bold;
    border: none;
    color: #fff;
}

ul.page-nav li, ul.page-nav li ul li {
    background-image: none;
    background-color: #000;
    opacity: 0.85;
    -moz-opacity: 0.85; /* older Gecko-based browsers */
    filter:alpha(opacity=85); /* For IE6&7 */
}

ul.page-nav li:hover,
ul.page-nav li.sfHover {
    border-bottom: 3px solid #999;
}

ul.page-nav li:hover,
ul.page-nav li.sfHover,
ul.page-nav a:focus, 
ul.page-nav a:hover, 
ul.page-nav a:active

{ 
    background: inherit;
    background-color: #999;
}

ul.page-nav ul a:focus, 
ul.page-nav ul a:hover, 
ul.page-nav ul a:active, 
ul.page-nav ul li.sfHover, 
ul.page-nav ul li:hover {
    background: none;
    border-bottom: none;
    background-color: #666;
    opacity: 70;
    -moz-opacity: 70; /* older Gecko-based browsers */
    filter:alpha(opacity=70); /* For IE6&7 */
}

.page-nav a, .page-nav a:visited {
    color: #fff;
    border: none;
}

.page-nav > li:hover > a > .sf-sub-indicator, .page-nav > li:active > a > .sf-sub-indicator,
.page-nav > li.sfHover > a > .sf-sub-indicator

{    
    background-image: url(/static/prophet/jquery/images/arrows-ffffff.png);
}


.page-nav > li > a > .sf-sub-indicator {    
    background-image: url(/static/prophet/jquery/images/arrows-cccccc.png);
}


.prop-summary {
    width: 80%;

}

 table.tablesorter tbody tr:nth-child(odd) td {
    background: #ddd;
}




table.tablesorter tr:hover td,
table.tablesorter tr:nth-child(odd):hover td {
    background: #ffc;
}


table.tablesorter tr:nth-child(odd) td.summary,
table.tablesorter tr td.summary {
    background: none;
}


dl.history {
    margin-right: 1em;
}
dl.history dt {
    border-top: 1px solid #ccc;
    padding-top: 0.5em;
    padding-bottom: 0.5em;

    color: #666; 
    width: 100%;
    font-size: 0.8em;
}

dl.history dt .source_info {
    float: right;
}

dl.history dt .created {
    width: 8em;
    display: inline-block;

}

dl.history dt .creator {
    width: 10em;
    display: inline-block;
}

dl.history dt .original_sequence_no {
    color: #666;
}

dl.history dt .original_sequence_no:after {
    content: " @ ";
}

dl.history dt .original_source_uuid {
    color: #666;
}


dl.history dd {
    margin-bottom: 0.5em; 
    font-size: 0.9em;
    margin-right: 0;
    }

dl.history dd ul li{ 
    list-style-type: none;
    padding: 0.5em;
    padding-left: 1.5em;
}

dl.history dd ul li:nth-child(odd) {
    background: #ddd;
    -webkit-border-radius: 0.4em;
    -moz-border-radius: 0.4em;
}


dl.history span.property {
    font-weight: bold;
    display: block;
    width: 8em;
    float: left; /* firefox can't text-align an inline block. nfc why */
    padding-right: 0.25em;
    text-align: right;
    overflow: hidden;
}

dl.history span.prose {
    color: #666;    
}

dl.history span.value.new,
dl.history span.value.old {
}

dl.history span.value.old {
}

ul.comments li {
    border-top: 1px solid #ccc;
    padding: 0.5em;
    margin-left: 1em;
    margin-right: 1em;
    border-bottom: 1px solid #ccc;
}

ul.comments .creator, 
ul.comments .created{
}


ul.comments .creator:after {
    content: " wrote:";
}

ul.comments li .content-pre {
    white-space: pre;
    overflow-x: auto;
}

ul.comments li .content {
    padding: 1em;
    margin-top: 1em;
    font-size: 0.9em;
    overflow-x: auto;

}

ul.comments li:nth-child(odd) {
    background: #ddd;
    -moz-border-radius: 0.4em;
    -webkit-border-radius: 0.4em;
}

table.tablesorter {
    width: 100%;
    background: #fff;
    border: none;
    position: relative;
    border-collapse: collapse;
    border-spacing: 0;
}

table.tablesorter thead tr th, table.tablesorter tfoot tr th {
    padding-right: 2em;
}
table.tablesorter td {
    border-bottom: 1px solid #ccc;
}

table.tablesorter tbody td {
 color: #555;
 font-weight: bold;
 height: 4em;
 padding-top: 3em;

}

table.tablesorter td.summary a, table.tablesorter td.id a {
 font-size: 1.6em;
 text-decoration: none;
 color: #700;
}

table.tablesorter td.summary a {
 display: block;
 padding-bottom: 2em;
 right: 0;
 background: clear;
 z-index: 50;
}



table.tablesorter td.id  {
    padding-top: 1.5em;
    text-align: right;
    width: 4em;
}
table.tablesorter td.id a {
    color: #bbb;
}

table.tablesorter td a:hover {
    text-decoration: underline;
}



table.tablesorter td.summary {
 margin-top: 0em;
 padding: 0;
 font-weight: normal;
 right:1.5em;
 overflow: hidden;
 margin-top: 0.5em;
 height: 1em;
 left: 6.75em;
 position: absolute;
 padding-bottom: 1.25em;
 border-bottom: none;

}

.pagesection {
 max-width: 130px;
 }

.pagesection .bar-wrapper {
 background: #ccc;
 height: 6px;
 margin-top: 4px;
 margin-bottom: 2px;
 -moz-border-radius: 4px;
 -webkit-border-radius: 4px;

}

.pagesection .bar {
 height: 6px;
 background: #900;
 -moz-border-radius: 4px;
 -webkit-border-radius: 4px;


 }

.pagesection li {
 font-size: 0.8em;
}

.pagesection li a {
  color: #000;
  padding-left: 0.1em;
  text-decoration: none;
 font-weight: bold;
}



div.overview {
 position: absolute;
 top: 80px;
 left: 160px;
 right: 20px;


}


div.overview h2 {
 padding-bottom: 0;
 font-weight: normal;
 border-bottom: 1px solid #ddd;
}

.sidebar {
  width: 120px;
  border-right: 1px solid #ddd;
  padding-right: 10px; 
}

.sidebar h2 { 
  padding-left: 0;
  font-size: 1.1em;
  font-weight: normal;
  border-bottom: 1px solid #ddd;
  padding-bottom: 5px;
  margin-bottom: 10px;
  padding-top: 0.5em;
}

.sidebar .pagesection {
 padding-bottom: 5px;

  border-bottom: 1px solid #ddd;

} 
